from lib.typing import *
from lib.parser import Lexer, Parser, TypeChecker

def read_file(filename):
    with open(filename, 'r') as file:
        return file.read()

def load_program(filename):
    lexer = Lexer(read_file(filename))
    lexer.process()
    parser = Parser(lexer.tokens)
    prog = parser.parse()
    checker = TypeChecker()
    checker.check(prog)
    return prog

def transform_program(prog):
    func  = prog.functions[0]
    stmts = []
    loop = None
    for stmt in func.body.statements:
        if loop == None:
            if isinstance(stmt, WhileStatement) and not contains_while(stmt.body.statements):
                loop = stmt
            elif not contains_while([stmt]):
                stmts.append(stmt)
            else:
                raise Exception
        else:
            if contains_while([stmt]):
                raise Exception
    return WhileLoopProgram(init=stmts, guard=loop.guard, body=loop.body.statements, args=func.args)

def contains_while(statements: List[Statement]):
    for stmt in statements:
        match stmt:
            case IfStatement(branches=branches):
                for (_, block) in branches:
                    if contains_while(block.statements):
                        return True
            case WhileStatement():
                return True


